/*
 * Copyright (c) 2004-2012 Zementis, Inc. All rights reserved.
 */
package com.zementis.adapa.sample.functions;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import com.zementis.adapa.extensions.Function;

/**
 * The {@link CalcSomething} function calculates something over several numeric arguments. In order to support
 * {@link Double}, {@link Float}, and {@link Long} arguments, it uses reflection to enable invocation of the appropriate
 * "doubleValue" method at runtime.
 */
public class CalcSomething implements Function<Double> {

    static final String NAME = "calc_something";

    private final Method[] doubleValueMethods;

    public CalcSomething(Class<?>... argumentTypes) {
        doubleValueMethods = new Method[argumentTypes.length];
        for (int i = 0; i < doubleValueMethods.length; i++) {
            try {
                doubleValueMethods[i] = argumentTypes[i].getMethod("doubleValue");
            } catch (SecurityException e) {
                // This should never happen
                throw new RuntimeException(e);
            } catch (NoSuchMethodException e) {
                // This should never happen
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    public Class<?> getFunctionType() {
        return Double.class;
    }

    @Override
    public String getName() {
        return NAME;
    }

    @Override
    public Double evaluate(Object... arguments) {
        double retVal = 0;
        for (int i = 0; i < arguments.length; i++) {
            // If any of the input values is null, then the function returns null
            if (arguments[i] == null) {
                return null;
            }
            try {
                retVal = retVal + (Double) doubleValueMethods[i].invoke(arguments[i]);
            } catch (IllegalArgumentException e) {
                // This should never happen
                throw new RuntimeException(e);
            } catch (IllegalAccessException e) {
                // This should never happen
                throw new RuntimeException(e);
            } catch (InvocationTargetException e) {
                // This should never happen
                throw new RuntimeException(e);
            }
        }

        return (retVal);
    }

}
