/*
 * Copyright (c) 2004-2012 Zementis, Inc. All rights reserved.
 */
package com.zementis.adapa.sample.functions;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.springframework.stereotype.Service;

import com.zementis.adapa.extensions.Function;

/**
 * Example of a custom function factory which creates functions that can computes something over a variable number (but
 * at least two) of numeric arguments.
 */
@Service
public class CalcSomethingFactory implements Function.Factory {

    private static final Set<Class<?>> SUPPORTED_ARGUMENT_TYPES = new HashSet<Class<?>>(
        Arrays.asList(new Class<?>[]{ Double.class, Float.class, Long.class, }));

    @Override
    public Function<?> createFunction(String functionName, Class<?>... argumentTypes) {
        // Check if the requested name matches our name
        if (!CalcSomething.NAME.equals(functionName)) {
            return null;
        }
        // The function requires at least two arguments
        if (argumentTypes.length < 2) {
            return null;
        }
        // All the argument types must be of one of the supported argument types (i.e. a numeric type)
        for (int i = 0; i < argumentTypes.length; i++) {
            if (!SUPPORTED_ARGUMENT_TYPES.contains(argumentTypes[i])) {
                return null;
            }
        }
        return new CalcSomething(argumentTypes);
    }

}
