/*
 * Copyright (c) 2004-2013 Zementis, Inc. All rights reserved.
 */
package com.zementis.adapa.sample.repository;

import java.io.InputStream;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.SerializationUtils;

import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;
import com.zementis.adapa.extensions.AssetRepository;

/**
 * A sample {@link AssetRepository} for MongoDB.
 * 
 */
public class MongoAssetRepository implements AssetRepository {

    private static final String ADAPA_DB = "adapa";

    private static final String ASSET_COLLECTION = "asset";

    private static final String ASSET_SOURCE = "source";

    private static final String ASSET_INFO = "info";

    private static final String IDENTIFIER = "identifier";

    private final DB database;

    public MongoAssetRepository(String hostname, int port) {
        MongoClient mongoClient;
        try {
            mongoClient = new MongoClient(hostname, port);
        } catch (Exception ex) {
            throw new RuntimeException("Could not create MongoClient");
        }
        database = mongoClient.getDB(ADAPA_DB);
    }

    public String addAsset(Serializable adapaAssetInfo, InputStream adapaAssetSource) {
        DBCollection table = database.getCollection(ASSET_COLLECTION);
        BasicDBObject document = new BasicDBObject();
        String identifier = String.valueOf(System.currentTimeMillis());
        document.put(IDENTIFIER, identifier);
        document.put(ASSET_INFO, SerializationUtils.serialize(adapaAssetInfo));
        String source;
        try {
            source = IOUtils.toString(adapaAssetSource);
        } catch (Exception ex) {
            source = "Could not create source from stream";
        }
        document.put(ASSET_SOURCE, source);
        table.insert(document);
        return identifier;
    }

    public Set<String> getAssetIds() {
        DBCollection table = database.getCollection(ASSET_COLLECTION);
        DBCursor cursor = table.find();
        Set<String> assetIds = new HashSet<String>();
        while (cursor.hasNext()) {
            assetIds.add(cursor.next().get(IDENTIFIER).toString());
        }
        return assetIds;
    }

    public Serializable getAssetInfo(String identifier) {
        DBCollection table = database.getCollection(ASSET_COLLECTION);
        BasicDBObject document = new BasicDBObject().append(IDENTIFIER, identifier);
        DBCursor cursor = table.find(document);
        if (cursor.hasNext()) {
            DBObject dbObject = cursor.next();
            byte[] adapaAssetBytes = (byte[]) dbObject.get(ASSET_INFO);
            return (Serializable) SerializationUtils.deserialize(adapaAssetBytes);
        }
        return null;
    }

    public InputStream getAssetSource(String identifier) {
        DBCollection table = database.getCollection(ASSET_COLLECTION);
        BasicDBObject document = new BasicDBObject().append(IDENTIFIER, identifier);
        DBCursor cursor = table.find(document);
        if (cursor.hasNext()) {
            DBObject dbObject = cursor.next();
            return IOUtils.toInputStream(dbObject.get(ASSET_SOURCE).toString());
        }
        return null;
    }

    public void removeAsset(String identifier) {
        DBCollection table = database.getCollection(ASSET_COLLECTION);
        BasicDBObject document = new BasicDBObject().append(IDENTIFIER, identifier);
        table.remove(document);
    }

}
